@@ -128,9 +128,31 @@ parasail_result_t* FNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_avx_256_64(s1, s1Len, matrix);
-    parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap);
+    /* declare local variables */
+    parasail_profile_t *profile = NULL;
+    parasail_result_t *result = NULL;
+
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+    PARASAIL_CHECK_NULL(matrix);
+    if (matrix->type == PARASAIL_MATRIX_TYPE_PSSM) {
+        PARASAIL_CHECK_NULL_PSSM_STATS(s1);
+    }
+    else {
+        PARASAIL_CHECK_NULL(s1);
+        PARASAIL_CHECK_GT0(s1Len);
+    }
+
+    /* initialize local variables */
+    profile = parasail_profile_create_stats_avx_256_64(s1, s1Len, matrix);
+    if (!profile) return NULL;
+    result = PNAME(profile, s2, s2Len, open, gap);
+
     parasail_profile_free(profile);
+
     return result;
 }
 
@@ -139,67 +161,157 @@ parasail_result_t* PNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap)
 {
+    /* declare local variables */
     int32_t i = 0;
     int32_t j = 0;
     int32_t k = 0;
-    const int s1Len = profile->s1Len;
-    int32_t end_query = s1Len-1;
-    int32_t end_ref = s2Len-1;
-    const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 4; /* number of values in vector unit */
-    const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    const int32_t offset = (s1Len - 1) % segLen;
-    const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-    __m256i* const restrict pvP  = (__m256i*)profile->profile64.score;
-    __m256i* const restrict pvPm = (__m256i*)profile->profile64.matches;
-    __m256i* const restrict pvPs = (__m256i*)profile->profile64.similar;
-    __m256i* const restrict pvE  = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvEM = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvES = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvEL = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvH  = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHM = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHS = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHL = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHMax  = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHMMax = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHSMax = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvHLMax = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvGapper = parasail_memalign___m256i(32, segLen);
-    __m256i* const restrict pvGapperL = parasail_memalign___m256i(32, segLen);
-    int64_t* const restrict boundary = parasail_memalign_int64_t(32, s2Len+1);
-    __m256i vGapO = _mm256_set1_epi64x_rpl(open);
-    __m256i vGapE = _mm256_set1_epi64x_rpl(gap);
-    const int64_t NEG_LIMIT = (-open < matrix->min ?
-        INT64_MIN + open : INT64_MIN - matrix->min) + 1;
-    const int64_t POS_LIMIT = INT64_MAX - matrix->max - 1;
-    __m256i vZero = _mm256_setzero_si256();
-    __m256i vOne = _mm256_set1_epi64x_rpl(1);
-    int64_t score = NEG_LIMIT;
+    int32_t s1Len = 0;
+    int32_t end_query = 0;
+    int32_t end_ref = 0;
+    const parasail_matrix_t *matrix = NULL;
+    int32_t segWidth = 0;
+    int32_t segLen = 0;
+    int32_t offset = 0;
+    int32_t position = 0;
+    __m256i* restrict pvP = NULL;
+    __m256i* restrict pvPm = NULL;
+    __m256i* restrict pvPs = NULL;
+    __m256i* restrict pvE = NULL;
+    __m256i* restrict pvEM = NULL;
+    __m256i* restrict pvES = NULL;
+    __m256i* restrict pvEL = NULL;
+    __m256i* restrict pvH = NULL;
+    __m256i* restrict pvHM = NULL;
+    __m256i* restrict pvHS = NULL;
+    __m256i* restrict pvHL = NULL;
+    __m256i* restrict pvHMax = NULL;
+    __m256i* restrict pvHMMax = NULL;
+    __m256i* restrict pvHSMax = NULL;
+    __m256i* restrict pvHLMax = NULL;
+    __m256i* restrict pvGapper = NULL;
+    __m256i* restrict pvGapperL = NULL;
+    int64_t* restrict boundary = NULL;
+    __m256i vGapO;
+    __m256i vGapE;
+    int64_t NEG_LIMIT = 0;
+    int64_t POS_LIMIT = 0;
+    __m256i vZero;
+    __m256i vOne;
+    int64_t score = 0;
     int64_t matches = 0;
     int64_t similar = 0;
     int64_t length = 0;
-    __m256i vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
-    __m256i vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
-    __m256i vSaturationCheckMin = vPosLimit;
-    __m256i vSaturationCheckMax = vNegLimit;
-    __m256i vNegInfFront = vZero;
+    __m256i vNegLimit;
+    __m256i vPosLimit;
+    __m256i vSaturationCheckMin;
+    __m256i vSaturationCheckMax;
+    __m256i vNegInfFront;
     __m256i vSegLenXgap;
-    __m256i vSegLen = _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(segLen), 8);
-#ifdef PARASAIL_TABLE
-    parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
-#else
-#ifdef PARASAIL_ROWCOL
-    parasail_result_t *result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
-#else
-    parasail_result_t *result = parasail_result_new_stats();
-#endif
-#endif
+    __m256i vSegLen;
+    parasail_result_t *result = NULL;
 
+    /* validate inputs */
+    PARASAIL_CHECK_NULL(profile);
+    PARASAIL_CHECK_NULL(profile->profile64.score);
+    PARASAIL_CHECK_NULL(profile->matrix);
+    PARASAIL_CHECK_GT0(profile->s1Len);
+    PARASAIL_CHECK_NULL(s2);
+    PARASAIL_CHECK_GT0(s2Len);
+    PARASAIL_CHECK_GE0(open);
+    PARASAIL_CHECK_GE0(gap);
+
+    /* initialize stack variables */
+    i = 0;
+    j = 0;
+    k = 0;
+    s1Len = profile->s1Len;
+    end_query = s1Len-1;
+    end_ref = s2Len-1;
+    matrix = profile->matrix;
+    segWidth = 4; /* number of values in vector unit */
+    segLen = (s1Len + segWidth - 1) / segWidth;
+    offset = (s1Len - 1) % segLen;
+    position = (segWidth - 1) - (s1Len - 1) / segLen;
+    pvP  = (__m256i*)profile->profile64.score;
+    pvPm = (__m256i*)profile->profile64.matches;
+    pvPs = (__m256i*)profile->profile64.similar;
+    vGapO = _mm256_set1_epi64x_rpl(open);
+    vGapE = _mm256_set1_epi64x_rpl(gap);
+    NEG_LIMIT = (-open < matrix->min ? INT64_MIN + open : INT64_MIN - matrix->min) + 1;
+    POS_LIMIT = INT64_MAX - matrix->max - 1;
+    vZero = _mm256_setzero_si256();
+    vOne = _mm256_set1_epi64x_rpl(1);
+    score = NEG_LIMIT;
+    matches = 0;
+    similar = 0;
+    length = 0;
+    vNegLimit = _mm256_set1_epi64x_rpl(NEG_LIMIT);
+    vPosLimit = _mm256_set1_epi64x_rpl(POS_LIMIT);
+    vSaturationCheckMin = vPosLimit;
+    vSaturationCheckMax = vNegLimit;
+    vNegInfFront = vZero;
+    vSegLen = _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(segLen), 8);
     vNegInfFront = _mm256_insert_epi64_rpl(vNegInfFront, NEG_LIMIT, 0);
     vSegLenXgap = _mm256_add_epi64(vNegInfFront,
             _mm256_slli_si256_rpl(_mm256_set1_epi64x_rpl(-segLen*gap), 8));
 
+    /* initialize result */
+#ifdef PARASAIL_TABLE
+    result = parasail_result_new_table3(segLen*segWidth, s2Len);
+#else
+#ifdef PARASAIL_ROWCOL
+    result = parasail_result_new_rowcol3(segLen*segWidth, s2Len);
+#else
+    result = parasail_result_new_stats();
+#endif
+#endif
+    if (!result) return NULL;
+
+    /* set known flags */
+    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
+        | PARASAIL_FLAG_STATS
+        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
+#ifdef PARASAIL_TABLE
+    result->flag |= PARASAIL_FLAG_TABLE;
+#endif
+#ifdef PARASAIL_ROWCOL
+    result->flag |= PARASAIL_FLAG_ROWCOL;
+#endif
+
+    /* initialize heap variables */
+    pvE  = parasail_memalign___m256i(32, segLen);
+    pvEM = parasail_memalign___m256i(32, segLen);
+    pvES = parasail_memalign___m256i(32, segLen);
+    pvEL = parasail_memalign___m256i(32, segLen);
+    pvH  = parasail_memalign___m256i(32, segLen);
+    pvHM = parasail_memalign___m256i(32, segLen);
+    pvHS = parasail_memalign___m256i(32, segLen);
+    pvHL = parasail_memalign___m256i(32, segLen);
+    pvHMax  = parasail_memalign___m256i(32, segLen);
+    pvHMMax = parasail_memalign___m256i(32, segLen);
+    pvHSMax = parasail_memalign___m256i(32, segLen);
+    pvHLMax = parasail_memalign___m256i(32, segLen);
+    pvGapper = parasail_memalign___m256i(32, segLen);
+    pvGapperL = parasail_memalign___m256i(32, segLen);
+    boundary = parasail_memalign_int64_t(32, s2Len+1);
+
+    /* validate heap variables */
+    if (!pvE) return NULL;
+    if (!pvEM) return NULL;
+    if (!pvES) return NULL;
+    if (!pvEL) return NULL;
+    if (!pvH) return NULL;
+    if (!pvHM) return NULL;
+    if (!pvHS) return NULL;
+    if (!pvHL) return NULL;
+    if (!pvHMax) return NULL;
+    if (!pvHMMax) return NULL;
+    if (!pvHSMax) return NULL;
+    if (!pvHLMax) return NULL;
+    if (!pvGapper) return NULL;
+    if (!pvGapperL) return NULL;
+    if (!boundary) return NULL;
+
     parasail_memset___m256i(pvHM, vZero, segLen);
     parasail_memset___m256i(pvHS, vZero, segLen);
     parasail_memset___m256i(pvHL, vZero, segLen);
@@ -519,15 +631,6 @@ parasail_result_t* PNAME(
     result->stats->matches = matches;
     result->stats->similar = similar;
     result->stats->length = length;
-    result->flag |= PARASAIL_FLAG_NW | PARASAIL_FLAG_SCAN
-        | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_4;
-#ifdef PARASAIL_TABLE
-    result->flag |= PARASAIL_FLAG_TABLE;
-#endif
-#ifdef PARASAIL_ROWCOL
-    result->flag |= PARASAIL_FLAG_ROWCOL;
-#endif
 
     parasail_free(boundary);
     parasail_free(pvGapperL);
